function muadd(varargin)
% add a run to the present one
% usage  muadd
% add a run to the present opens a gui
%
% usage  muload(n)
% adds NRUN + n
%
% usage  muload(zp,nrun)
%
% Loads data from a run, using the path and filename information
%
% zp is the number of characters for nrun to be zero padded 
% nrun is the run number (integer)
% It will be saved in the integer variable MU_DATA.NRUN
% Raw data are stored in global structure MU_DATA.DATA
% Other fields are
% RESNS (ns/bin)
% START_TIME, START_DATE, STOP_TIME, STOP_DATE
% TITLE, SAMPLE, ORIENTATION, FIELD , TEMPERATURE, COMMENTS
    
    
global MU_GEAR MU_PATH MU_TABLE MU_DATA
global MU_ADD
if ~isfield(MU_DATA,'NRUN'), 
    muload(varargin) 
else
    disp(['Adding data to run ' num2str(MU_DATA.NRUN) ': ' MU_DATA.TITLE] )
    narg=size(varargin,2);
    localdir='';
    filename='';
    pathname='';
    
    switch narg
      case 0
        if isstruct(MU_PATH)
            localdir=pwd;
            cd(MU_PATH.DATA)
            [filename pathname s]=uigetfile(['*.' ...
                                MU_GEAR.FILENAME_SUFFIX],...
                                            'ADD MUSR DATA RUN');
        else 
            [filename pathname s]=uigetfile(['*.' ...
                                MU_GEAR.FILENAME_SUFFIX],...
                                            'ADD MUSR DATA RUN');
        end
        if ~isempty(localdir)
            cd(localdir)
        end
        if ~s
            filename='';
        end
        if  ~isempty(filename)
            if ~strcmp(pathname,MU_PATH.DATA) 
                MU_PATH.DATA=pathname;   % change data path
            end
            file=filename;
            filename=[MU_PATH.DATA filename];
            npre=length(MU_GEAR.FILENAME_PREFIX);
            nsuf=length(MU_GEAR.FILENAME_SUFFIX);
            if strncmp(MU_GEAR.FILENAME_PREFIX,...
                       file,npre) & ...
                    strcmp(MU_GEAR.FILENAME_SUFFIX,file(end-nsuf+1: ...
                                                        end)) & ...
                    ~isempty(str2num(...
                        file(npre+1:end-nsuf)))
                nrun=str2num(file(npre+1:end-nsuf));
                MU_ADD=[MU_ADD nrun];
                zeropad='00000000000000000';
                zp=length(file(npre+1:end-nsuf-1))-...
                   length(num2str(nrun));
                zeropad=zeropad(1:zp);
            else 
                disp([MU_GEAR.FILENAME_PREFIX ':' ...
                      file(1:npre)])
                disp([MU_GEAR.FILENAME_SUFFIX ':' ...
                      filename(end-nsuf+1:end)])
                disp(file(npre+1:end-nsuf))
                disp('Prefix or suffix do not match. Use mufilename')
                filename='';
            end
        
        end
      case 1
        n=length(num2str(MU_DATA.NRUN));
        nrun=MU_DATA.NRUN+cell2mat(varargin(1));
        MU_ADD=[MU_ADD nrun];
        zeropad=MU_DATA.ZEROPAD;        
        if length(num2str(nrun))>n  % increases number of digits
            zeropad=zeropad(1:end-1);  % decrease zeropadding
        end
        filename=strcat(MU_PATH.DATA,MU_GEAR.FILENAME_PREFIX,...
                        zeropad,num2str(nrun),...
                        '.',MU_GEAR.FILENAME_SUFFIX);
        
      case 2
        zp=cell2mat(varargin(1));
        nrun=cell2mat(varargin(2));
        if isnumeric(nrun)
            zeroes='0000000000000000000';
            s=int2str(nrun);
            zeropad=zeroes(1:zp-length(s));
            if length(s)<zp, s=[zeropad s]; end
            filename=strcat(MU_PATH.DATA,MU_GEAR.FILENAME_PREFIX,...
                        s,'.',MU_GEAR.FILENAME_SUFFIX);
            MU_ADD=[MU_ADD nrun];
        else
            disp('nrun must be a number!')
        end
    end
    if ~isempty(filename)
        if exist(filename)
            MU_TABLE.TAUMU=2197.03;  % ns
            switch MU_GEAR.FACILITY
              case 'PSI'
                [histos dt head]=psiload(filename);
              case 'ISIS'
                switch MU_GEAR.DATA_TYPE
                  case 'MCS'
                    [histos dt head]=isisload(filename);
                  case 'NEXUS'
                    [histos dt head]=nxsload(filename);
                end 
            end
            if size(histos)==size(MU_DATA.DATA)&...
                    dt/1000==MU_DATA.RESNS
            MU_DATA.DATA=MU_DATA.DATA+histos; %
            ntot=sum(sum(MU_DATA.DATA(max(MU_DATA.NT0):end,:)));
            disp(['Run ' num2str(nrun) ' added: ' MU_DATA.TITLE(1:44) 'Total '... 
                  num2str(ntot/1e6,3) ' Mev'])
            else
                disp(['DATA RUN ' nun2str(nrun) ' NOT ADDDED: INCOMPATIBLE']) 
                disp(['Histos size: ' num2str(size(histos)) ' cfr. ' ...
                                    'run ' num2str(MU_DATA.NRUN) ...
                                    ': ' num2str(size(MU_DATA.DATA))])
                disp(['Resolution: ' num2str(dt/1000) 'ns cfr. ' ...
                                    'run ' num2str(MU_DATA.NRUN) ...
                                    ': ' num2str(size(MU_DATA.RESNS))])
            end
        else
            disp([filename ' not found'])
        end
    else
        disp('No run selected')
    end
end