function mufit
% usage mufit
%
% This function executes the fminuit fit
% It requires prior execution of
% mu_initialize_models 

global MU_MODEL MU_DATA  
global MU_MODEL_F
global MU_ADD
global MU_PATH

if isfield(MU_DATA,'NRUN')
    muasymmetry;
    
    data=[MU_DATA.TIME(MU_MODEL.RANGE(1):MU_MODEL.RANGE(2)); ...
          MU_DATA.ASYMMETRY(MU_MODEL.RANGE(1):MU_MODEL.RANGE(2)); ...
          MU_DATA.ASYMERROR(MU_MODEL.RANGE(1):MU_MODEL.RANGE(2))];
    %size(data)
    %
    % set parnames
    [znpin stpbnd parname pf]=mupar(1); % use MU_MODEL
    parnames=[];
    for k=1:length(parname);
        parnames=[parnames ' ' cell2mat(parname(k))];
    end
    parnames=parnames(2:end); % remove starting blank
    %
    % recalculate array SHARE which redistributes parameters
    % must have as many elements as the total number of parameters
    % in Fit Parameter Menu
    % but share(j) is the index of par inside fminuit 
    % where += parameters do not appear
    shareglobal=[]; % from scratch at the beginning; it contains
                    % all indices
    par=0;
    % FIRST PASSAGE: ~!=, + are just counted in
    for k=1:MU_MODEL.COMPONENTNUMBER
        share=[]; % this is the component SHARE array for the
                  % present component; it contains only indices for
                  % the present component 
        for j=1:MU_MODEL.COMPONENT(k).PARAMETERNUMBER
            switch MU_MODEL.COMPONENT(k).PARAMETER(j).FLAG
              case {'~','!'} % also a regular fminuit parameter
                par=par+1; % index is incremented only for ~! parameters
                share=[share par]; % for present component
                shareglobal=[shareglobal par]; % total
                MU_MODEL.COMPONENT(k).PARAMETER(j).MENU2MINUIT=...
                    ['p(' num2str(par) ')']; % new 1.05 version
              case '+' % linear relation must be reassigned and
                       % stored in 
                       % MU_MODEL.COMPONENT(k).PARAMETER(kp).MENU2MINUIT
                         % at a later passage (since reference
                         % could be to a subsequent parameter,
                         % shareglobal must be completed)
                share=[share 0]; % for present component
                shareglobal=[shareglobal 0]; % total
              case '=' % must be a preceding parameter
                parshared=MU_MODEL.COMPONENT(k).PARAMETER(j).SHARE;
                share=[share shareglobal(parshared)]; % for present component
                shareglobal=[shareglobal shareglobal(parshared)]; % total
                MU_MODEL.COMPONENT(k).PARAMETER(j).MENU2MINUIT=...
                    ['p(' num2str(shareglobal(parshared)) ')']; 
                % new 1.05 version
            end
        end
        %       disp (['comp=' num2str(k) ': ' num2str(share)])
        MU_MODEL.COMPONENT(k).SHARE=share; % remains in 1.05
    end
    % SECOND PASSAGE: recalculate +
    for k=1:MU_MODEL.COMPONENTNUMBER
         for j=1:MU_MODEL.COMPONENT(k).PARAMETERNUMBER
            switch MU_MODEL.COMPONENT(k).PARAMETER(j).FLAG
                case '+' % linear relation must be reassigned and
                         % stored in 
                         % MU_MODEL.COMPONENT(k).PARAMETER(kp).MENU2MINUIT
                  [OK open close par]=muparse(...
                      MU_MODEL.COMPONENT(k).PARAMETER(j).LINREL);
                  % get open = indices of 'p('
                  %     close = indices of ')'
                  %     par = indices of SOURCE parameters
                  %           in Menu (shareglobal) list
                  string=MU_MODEL.COMPONENT(k).PARAMETER(j).LINREL;
                  % this is the original relation, already verified
                  % by muparse when it was typed in
                  % It will be rewritten in string
                  for l=1:length(open)
                      % first make space
                      if close(l)-open(l)==3 % only one digit per index
                          string(open(l)+2:end+1)=['  ' ...
                                              string(close(l):end)];

                          close(l)=close(l)+1; % shift forward
                          for ll=l+1:length(open) % shift forward
                                                  % all the following
                              open(ll)=open(ll)+1;
                              close(ll)=close(ll)+1;
                          end
                          % make room for two (blank) digits and
                          % copy rest                          
                      else % already two digits per index
                         string(open(l)+2:close(l)-1)='  '; % blank them 
                      end 
                      % then write reallocated index
                      nback=2; % index is one digit
                      if fix(shareglobal(par(l))/10) 
                          nback=1; % index is two digits
                      end
                      string(open(l)+2:close(l)-nback)=...
                          num2str(shareglobal(par(l)));
                      % write the fminuit index
                      % between p(_  and _) or __), depending on
                      % number of digits 
                  end
                  % finaly store in MENU2MINUIT
                  MU_MODEL.COMPONENT(k).PARAMETER(j).MENU2MINUIT=...
                      string; % new 1.05 version
            end
        end
    end
     
    % execute FMINUIT
    [znpout,znepout,chi]=fminuit('mufcn','mnplot',znpin,data, ...
                                 '-s',stpbnd,'-c','mini;hesse;exit',...
                                 '-n',parnames);
    nu=length(data(1,:))-(length(znpin)-pf); % degrees of freedom
        
    par=0;
    MU_MODEL_F=MU_MODEL; % copy all, to start with, including new MENU2MINUIT
    if ~isempty(MU_ADD)
        MU_MODEL_F.ADD=MU_ADD;
    else
        MU_MODEL_F.ADD=[];
    end
    % add a few things which are not included in MU_MODEL
    % best fit details
    MU_MODEL_F.PAR=znpout;
    MU_MODEL_F.EPAR=znepout;
    MU_MODEL_F.NU=nu;
    MU_MODEL_F.CHI=chi;
    MU_MODEL_F.DATE=date;
    string=datestr(clock);
    MU_MODEL_F.TIME=string(13:20);
    % data details
    MU_MODEL_F.NRUN=MU_DATA.NRUN;
    MU_MODEL_F.DATAPATH=MU_PATH.DATA;
    MU_MODEL_F.SAMPLE=MU_DATA.SAMPLE;
    MU_MODEL_F.TEMPERATURE=MU_DATA.TEMPERATURE;
    MU_MODEL_F.FIELD=MU_DATA.FIELD;
    MU_MODEL_F.ORIENTATION=MU_DATA.ORIENTATION;
    % now update best fit values as parameter staring guess 
    for k=1:MU_MODEL.COMPONENTNUMBER
        pn=MU_MODEL.COMPONENT(k).PARAMETERNUMBER;
        component=MU_MODEL.COMPONENT(k).NAME;
        par=0;
        for j=1:pn %MU_MODEL.COMPONENT(k).PARAMETERNUMBER
            par=par+1;
            share=MU_MODEL_F.COMPONENT(k).SHARE;
            if MU_MODEL.COMPONENT(k).PARAMETER(j).FLAG=='~'|...
                    MU_MODEL.COMPONENT(k).PARAMETER(j).FLAG=='!'
                MU_MODEL_F.COMPONENT(k).PARAMETER(j).VALUE=...
                    znpout(share(par));
                MU_MODEL_F.COMPONENT(k).PARAMETER(j).ERROR= ...
                    znepout(share(par));
            else
                MU_MODEL_F.COMPONENT(k).PARAMETER(j).VALUE=0;
                MU_MODEL_F.COMPONENT(k).PARAMETER(j).ERROR=0;
                % not used: it is recalculted from its ingredients
            end
            
        end
    end
    
    musummary(chi,nu,data(1,1),data(1,end),data(1,2)-data(1,1));
    muplotfit;
    musavemodel(1) % dummy argument means save MU_MODEL_F
                   %    string=['To save fit results as initial guess type',...
                   %'    muupdatemodel'];
                   %    string=strvcat(string,['To check fit on different range type e.g.',...
                   %                        '    muplotfit(2,[1 500])']);
                   %    disp(string)
else
    disp('You must first load your data! ')
end
